@@ -40,6 +40,7 @@ G_BEGIN_DECLS
 #include "hdy-header-group.h"
 #include "hdy-keypad.h"
 #include "hdy-leaflet.h"
+#include "hdy-main.h"
 #include "hdy-navigation-direction.h"
 #include "hdy-preferences-group.h"
 #include "hdy-preferences-page.h"
@@ -48,6 +49,7 @@ G_BEGIN_DECLS
 #include "hdy-search-bar.h"
 #include "hdy-squeezer.h"
 #include "hdy-swipe-group.h"
+#include "hdy-swipe-tracker.h"
 #include "hdy-swipeable.h"
 #include "hdy-title-bar.h"
 #include "hdy-value-object.h"
