@@ -12,8 +12,8 @@
 #include "hdy-animation-private.h"
 #include "hdy-carousel-box-private.h"
 #include "hdy-navigation-direction.h"
-#include "hdy-swipe-tracker-private.h"
-#include "hdy-swipeable-private.h"
+#include "hdy-swipe-tracker.h"
+#include "hdy-swipeable.h"
 
 #include <math.h>
 
@@ -180,36 +180,42 @@ hdy_carousel_switch_child (HdySwipeable *swipeable,
 }
 
 static void
-hdy_carousel_begin_swipe (HdySwipeable           *swipeable,
-                          HdyNavigationDirection  direction,
-                          gboolean                direct)
+begin_swipe_cb (HdySwipeTracker        *tracker,
+                HdyNavigationDirection  direction,
+                gboolean                direct,
+                HdyCarousel            *self)
 {
-  HdyCarousel *self = HDY_CAROUSEL (swipeable);
-
   hdy_carousel_box_stop_animation (self->scrolling_box);
 }
 
 static void
-hdy_carousel_update_swipe (HdySwipeable *swipeable,
-                           gdouble       value)
+update_swipe_cb (HdySwipeTracker *tracker,
+                 gdouble          progress,
+                 HdyCarousel     *self)
 {
-  HdyCarousel *self = HDY_CAROUSEL (swipeable);
-
-  hdy_carousel_box_set_position (self->scrolling_box, value);
+  hdy_carousel_box_set_position (self->scrolling_box, progress);
 }
 
 static void
-hdy_carousel_end_swipe (HdySwipeable *swipeable,
-                        gint64        duration,
-                        gdouble       to)
+end_swipe_cb (HdySwipeTracker *tracker,
+              gint64           duration,
+              gdouble          to,
+              HdyCarousel     *self)
 {
-  HdyCarousel *self = HDY_CAROUSEL (swipeable);
   GtkWidget *child;
 
   child = hdy_carousel_box_get_page_at_position (self->scrolling_box, to);
   hdy_carousel_box_scroll_to (self->scrolling_box, child, duration);
 }
 
+static HdySwipeTracker *
+hdy_carousel_get_swipe_tracker (HdySwipeable *swipeable)
+{
+  HdyCarousel *self = HDY_CAROUSEL (swipeable);
+
+  return self->tracker;
+}
+
 static gdouble
 hdy_carousel_get_distance (HdySwipeable *swipeable)
 {
@@ -218,16 +224,6 @@ hdy_carousel_get_distance (HdySwipeable *swipeable)
   return hdy_carousel_box_get_distance (self->scrolling_box);
 }
 
-static void
-hdy_carousel_get_range (HdySwipeable *swipeable,
-                        gdouble      *lower,
-                        gdouble      *upper)
-{
-  HdyCarousel *self = HDY_CAROUSEL (swipeable);
-
-  hdy_carousel_box_get_range (self->scrolling_box, lower, upper);
-}
-
 static gdouble *
 hdy_carousel_get_snap_points (HdySwipeable *swipeable,
                               gint         *n_snap_points)
@@ -917,11 +913,8 @@ static void
 hdy_carousel_swipeable_init (HdySwipeableInterface *iface)
 {
   iface->switch_child = hdy_carousel_switch_child;
-  iface->begin_swipe = hdy_carousel_begin_swipe;
-  iface->update_swipe = hdy_carousel_update_swipe;
-  iface->end_swipe = hdy_carousel_end_swipe;
+  iface->get_swipe_tracker = hdy_carousel_get_swipe_tracker;
   iface->get_distance = hdy_carousel_get_distance;
-  iface->get_range = hdy_carousel_get_range;
   iface->get_snap_points = hdy_carousel_get_snap_points;
   iface->get_progress = hdy_carousel_get_progress;
   iface->get_cancel_progress = hdy_carousel_get_cancel_progress;
@@ -1161,6 +1154,11 @@ hdy_carousel_init (HdyCarousel *self)
 
   self->tracker = hdy_swipe_tracker_new (HDY_SWIPEABLE (self));
   hdy_swipe_tracker_set_allow_mouse_drag (self->tracker, TRUE);
+
+  g_signal_connect_object (self->tracker, "begin-swipe", G_CALLBACK (begin_swipe_cb), self, 0);
+  g_signal_connect_object (self->tracker, "update-swipe", G_CALLBACK (update_swipe_cb), self, 0);
+  g_signal_connect_object (self->tracker, "end-swipe", G_CALLBACK (end_swipe_cb), self, 0);
+
   self->can_scroll = TRUE;
 }
 
@@ -1289,7 +1287,7 @@ hdy_carousel_scroll_to_full (HdyCarousel *self,
 
   hdy_carousel_box_scroll_to (self->scrolling_box, widget,
                                duration);
-  hdy_swipeable_emit_switch_child (HDY_SWIPEABLE (self), n, duration);
+  hdy_swipeable_emit_child_switched (HDY_SWIPEABLE (self), n, duration);
 }
 
 /**
