@@ -10,7 +10,7 @@
 
 #include "hdy-deck.h"
 #include "hdy-stackable-box-private.h"
-#include "hdy-swipeable-private.h"
+#include "hdy-swipeable.h"
 
 /**
  * SECTION:hdy-deck
@@ -21,6 +21,12 @@
  * #GtkStack. The children are strictly ordered and can be navigated using
  * swipe gestures.
  *
+ * The “over” and “under” transitions can draw their shadow on top of the
+ * window's transparent areas, like the rounded corners. This is a side-effect
+ * of allowing shadows to be drawn on top of OpenGL areas. It can be mitigated
+ * by using #HdyWindow or #HdyApplicationWindow as they will crop anything drawn
+ * beyond the rounded corners.
+ *
  * # CSS nodes
  *
  * #HdyDeck has a single CSS node with name deck.
@@ -366,7 +372,7 @@ hdy_deck_set_interpolate_size (HdyDeck  *self,
  * hdy_deck_get_interpolate_size:
  * @self: a #HdyDeck
  *
- * Returns wether the #HdyDeck is set up to interpolate between
+ * Returns whether the #HdyDeck is set up to interpolate between
  * the sizes of children on page switch.
  *
  * Returns: %TRUE if child sizes are interpolated
@@ -769,27 +775,10 @@ hdy_deck_switch_child (HdySwipeable *swipeable,
   hdy_stackable_box_switch_child (HDY_GET_HELPER (swipeable), index, duration);
 }
 
-static void
-hdy_deck_begin_swipe (HdySwipeable           *swipeable,
-                      HdyNavigationDirection  direction,
-                      gboolean                direct)
+static HdySwipeTracker *
+hdy_deck_get_swipe_tracker (HdySwipeable *swipeable)
 {
-  hdy_stackable_box_begin_swipe (HDY_GET_HELPER (swipeable), direction, direct);
-}
-
-static void
-hdy_deck_update_swipe (HdySwipeable *swipeable,
-                       gdouble       value)
-{
-  hdy_stackable_box_update_swipe (HDY_GET_HELPER (swipeable), value);
-}
-
-static void
-hdy_deck_end_swipe (HdySwipeable *swipeable,
-                    gint64        duration,
-                    gdouble       to)
-{
-  hdy_stackable_box_end_swipe (HDY_GET_HELPER (swipeable), duration, to);
+  return hdy_stackable_box_get_swipe_tracker (HDY_GET_HELPER (swipeable));
 }
 
 static gdouble
@@ -798,12 +787,11 @@ hdy_deck_get_distance (HdySwipeable *swipeable)
   return hdy_stackable_box_get_distance (HDY_GET_HELPER (swipeable));
 }
 
-static void
-hdy_deck_get_range (HdySwipeable *swipeable,
-                       gdouble      *lower,
-                       gdouble      *upper)
+static gdouble *
+hdy_deck_get_snap_points (HdySwipeable *swipeable,
+                          gint         *n_snap_points)
 {
-  hdy_stackable_box_get_range (HDY_GET_HELPER (swipeable), lower, upper);
+  return hdy_stackable_box_get_snap_points (HDY_GET_HELPER (swipeable), n_snap_points);
 }
 
 static gdouble
@@ -1069,11 +1057,9 @@ static void
 hdy_deck_swipeable_init (HdySwipeableInterface *iface)
 {
   iface->switch_child = hdy_deck_switch_child;
-  iface->begin_swipe = hdy_deck_begin_swipe;
-  iface->update_swipe = hdy_deck_update_swipe;
-  iface->end_swipe = hdy_deck_end_swipe;
+  iface->get_swipe_tracker = hdy_deck_get_swipe_tracker;
   iface->get_distance = hdy_deck_get_distance;
-  iface->get_range = hdy_deck_get_range;
+  iface->get_snap_points = hdy_deck_get_snap_points;
   iface->get_progress = hdy_deck_get_progress;
   iface->get_cancel_progress = hdy_deck_get_cancel_progress;
 }
