@@ -12,6 +12,7 @@
 
 #include <gtk/gtk.h>
 #include "hdy-navigation-direction.h"
+#include "hdy-swipe-tracker.h"
 
 G_BEGIN_DECLS
 
@@ -87,18 +88,11 @@ void             hdy_stackable_box_set_child_allow_visible (HdyStackableBox *sel
 void             hdy_stackable_box_switch_child (HdyStackableBox *self,
                                                  guint            index,
                                                  gint64           duration);
-void             hdy_stackable_box_begin_swipe (HdyStackableBox        *self,
-                                                HdyNavigationDirection  direction,
-                                                gboolean                direct);
-void             hdy_stackable_box_update_swipe (HdyStackableBox *self,
-                                                 gdouble          value);
-void             hdy_stackable_box_end_swipe (HdyStackableBox *self,
-                                              gint64           duration,
-                                              gdouble          to);
+
+HdySwipeTracker *hdy_stackable_box_get_swipe_tracker (HdyStackableBox *self);
 gdouble          hdy_stackable_box_get_distance (HdyStackableBox *self);
-void             hdy_stackable_box_get_range (HdyStackableBox *self,
-                                              gdouble         *lower,
-                                              gdouble         *upper);
+gdouble         *hdy_stackable_box_get_snap_points (HdyStackableBox *self,
+                                                    gint            *n_snap_points);
 gdouble          hdy_stackable_box_get_progress (HdyStackableBox *self);
 gdouble          hdy_stackable_box_get_cancel_progress (HdyStackableBox *self);
 
