@@ -24,11 +24,10 @@
 #define DRAG_THRESHOLD_DISTANCE 5
 
 /**
- * PRIVATE:hdy-swipe-tracker
+ * SECTION:hdy-swipe-tracker
  * @short_description: Swipe tracker used in #HdyCarousel and #HdyLeaflet
  * @title: HdySwipeTracker
  * @See_also: #HdyCarousel, #HdyDeck, #HdyLeaflet, #HdySwipeable
- * @stability: Private
  *
  * The HdySwipeTracker object can be used for implementing widgets with swipe
  * gestures. It supports touch-based swipes, pointer dragging, and touchpad
@@ -38,7 +37,7 @@
  * If they expect to use horizontal orientation, #HdySwipeTracker:reversed
  * property can be used for supporting RTL text direction.
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 
 typedef enum {
@@ -46,6 +45,7 @@ typedef enum {
   HDY_SWIPE_TRACKER_STATE_PENDING,
   HDY_SWIPE_TRACKER_STATE_SCROLLING,
   HDY_SWIPE_TRACKER_STATE_FINISHING,
+  HDY_SWIPE_TRACKER_STATE_REJECTED,
 } HdySwipeTrackerState;
 
 struct _HdySwipeTracker
@@ -58,6 +58,9 @@ struct _HdySwipeTracker
   gboolean allow_mouse_drag;
   GtkOrientation orientation;
 
+  gint start_x;
+  gint start_y;
+
   guint32 prev_time;
   gdouble velocity;
 
@@ -90,6 +93,15 @@ enum {
 
 static GParamSpec *props[LAST_PROP];
 
+enum {
+  SIGNAL_BEGIN_SWIPE,
+  SIGNAL_UPDATE_SWIPE,
+  SIGNAL_END_SWIPE,
+  SIGNAL_LAST_SIGNAL,
+};
+
+static guint signals[SIGNAL_LAST_SIGNAL];
+
 static void
 reset (HdySwipeTracker *self)
 {
@@ -100,6 +112,9 @@ reset (HdySwipeTracker *self)
   self->initial_progress = 0;
   self->progress = 0;
 
+  self->start_x = 0;
+  self->start_y = 0;
+
   self->prev_time = 0;
   self->velocity = 0;
 
@@ -109,14 +124,41 @@ reset (HdySwipeTracker *self)
     gtk_grab_remove (GTK_WIDGET (self->swipeable));
 }
 
+static void
+get_range (HdySwipeTracker *self,
+           gdouble         *first,
+           gdouble         *last)
+{
+  g_autofree gdouble *points = NULL;
+  gint n;
+
+  points = hdy_swipeable_get_snap_points (self->swipeable, &n);
+
+  *first = points[0];
+  *last = points[n - 1];
+}
+
 static void
 gesture_prepare (HdySwipeTracker        *self,
                  HdyNavigationDirection  direction)
 {
+  GdkRectangle rect;
+
   if (self->state != HDY_SWIPE_TRACKER_STATE_NONE)
     return;
 
-  hdy_swipeable_begin_swipe (self->swipeable, direction, TRUE);
+  hdy_swipeable_get_swipe_area (self->swipeable, &rect);
+
+  if (self->start_x < rect.x ||
+      self->start_x >= rect.x + rect.width ||
+      self->start_y < rect.y ||
+      self->start_y >= rect.y + rect.height) {
+    self->state = HDY_SWIPE_TRACKER_STATE_REJECTED;
+
+    return;
+  }
+
+  hdy_swipe_tracker_emit_begin_swipe (self, direction, TRUE);
 
   self->initial_progress = hdy_swipeable_get_progress (self->swipeable);
   self->progress = self->initial_progress;
@@ -156,7 +198,7 @@ gesture_update (HdySwipeTracker *self,
   if (time != self->prev_time)
     self->velocity = delta / (time - self->prev_time);
 
-  hdy_swipeable_get_range (self->swipeable, &first_point, &last_point);
+  get_range (self, &first_point, &last_point);
 
   progress = self->progress + delta;
   progress = CLAMP (progress, first_point, last_point);
@@ -166,7 +208,7 @@ gesture_update (HdySwipeTracker *self,
 
   self->progress = progress;
 
-  hdy_swipeable_update_swipe (self->swipeable, progress);
+  hdy_swipe_tracker_emit_update_swipe (self, progress);
 
   self->prev_time = time;
 }
@@ -241,7 +283,7 @@ gesture_end (HdySwipeTracker *self,
   if (self->progress != end_progress)
     duration = CLAMP (duration, MIN_ANIMATION_DURATION, MAX_ANIMATION_DURATION);
 
-  hdy_swipeable_end_swipe (self->swipeable, duration, end_progress);
+  hdy_swipe_tracker_emit_end_swipe (self, duration, end_progress);
 
   if (self->cancelled)
     reset (self);
@@ -269,6 +311,9 @@ drag_begin_cb (HdySwipeTracker *self,
 {
   if (self->state != HDY_SWIPE_TRACKER_STATE_NONE)
     gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
+
+  self->start_x = start_x;
+  self->start_y = start_y;
 }
 
 static void
@@ -293,6 +338,11 @@ drag_update_cb (HdySwipeTracker *self,
 
   is_offset_vertical = (ABS (offset_y) > ABS (offset_x));
 
+  if (self->state == HDY_SWIPE_TRACKER_STATE_REJECTED) {
+    gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
+    return;
+  }
+
   if (self->state == HDY_SWIPE_TRACKER_STATE_NONE) {
     if (is_vertical == is_offset_vertical)
       gesture_prepare (self, offset > 0 ? HDY_NAVIGATION_DIRECTION_FORWARD : HDY_NAVIGATION_DIRECTION_BACK);
@@ -306,7 +356,7 @@ drag_update_cb (HdySwipeTracker *self,
     gdouble first_point, last_point;
     gboolean is_overshooting;
 
-    hdy_swipeable_get_range (self->swipeable, &first_point, &last_point);
+    get_range (self, &first_point, &last_point);
 
     drag_distance = sqrt (offset_x * offset_x + offset_y * offset_y);
     is_overshooting = (offset < 0 && self->progress <= first_point) ||
@@ -338,6 +388,13 @@ drag_end_cb (HdySwipeTracker *self,
 
   distance = hdy_swipeable_get_distance (self->swipeable);
 
+  if (self->state == HDY_SWIPE_TRACKER_STATE_REJECTED) {
+    gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
+
+    reset (self);
+    return;
+  }
+
   if (self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING) {
     gesture_cancel (self, distance);
     gtk_gesture_set_state (self->touch_gesture, GTK_EVENT_SEQUENCE_DENIED);
@@ -398,13 +455,29 @@ handle_scroll_event (HdySwipeTracker *self,
     return GDK_EVENT_PROPAGATE;
   }
 
+  if (self->state == HDY_SWIPE_TRACKER_STATE_REJECTED) {
+    if (gdk_event_is_scroll_stop_event (event))
+      reset (self);
+
+    return GDK_EVENT_PROPAGATE;
+  }
+
   if (self->state == HDY_SWIPE_TRACKER_STATE_NONE) {
     if (gdk_event_is_scroll_stop_event (event))
       return GDK_EVENT_PROPAGATE;
 
     if (is_vertical == is_delta_vertical) {
-      if (!capture)
+      if (!capture) {
+        GtkWidget *widget = gtk_get_event_widget (event);
+        gdouble event_x, event_y;
+
+        gdk_event_get_coords (event, &event_x, &event_y);
+        gtk_widget_translate_coordinates (widget, GTK_WIDGET (self->swipeable),
+                                          event_x, event_y,
+                                          &self->start_x, &self->start_y);
+
         gesture_prepare (self, delta > 0 ? HDY_NAVIGATION_DIRECTION_FORWARD : HDY_NAVIGATION_DIRECTION_BACK);
+      }
     } else {
       self->is_scrolling = TRUE;
       return GDK_EVENT_PROPAGATE;
@@ -415,7 +488,7 @@ handle_scroll_event (HdySwipeTracker *self,
     gboolean is_overshooting;
     gdouble first_point, last_point;
 
-    hdy_swipeable_get_range (self->swipeable, &first_point, &last_point);
+    get_range (self, &first_point, &last_point);
 
     is_overshooting = (delta < 0 && self->progress <= first_point) ||
                       (delta > 0 && self->progress >= last_point);
@@ -518,7 +591,7 @@ static gboolean
 captured_event_cb (HdySwipeable *swipeable,
                    GdkEvent     *event)
 {
-  HdySwipeTracker *self = g_object_get_data (G_OBJECT (swipeable), "swipe-tracker");
+  HdySwipeTracker *self = hdy_swipeable_get_swipe_tracker (swipeable);
 
   g_assert (HDY_IS_SWIPE_TRACKER (self));
 
@@ -558,8 +631,6 @@ hdy_swipe_tracker_constructed (GObject *object)
 
   g_signal_connect_object (self->swipeable, "event", G_CALLBACK (handle_event_cb), self, G_CONNECT_SWAPPED);
 
-  g_object_set_data (G_OBJECT (self->swipeable), "swipe-tracker", self);
-
   /*
    * HACK: GTK3 has no other way to get events on capture phase.
    * This is a reimplementation of _gtk_widget_set_captured_event_handler(),
@@ -582,7 +653,6 @@ hdy_swipe_tracker_dispose (GObject *object)
   if (self->touch_gesture)
     g_signal_handlers_disconnect_by_data (self->touch_gesture, self);
 
-  g_object_set_data (G_OBJECT (self->swipeable), "swipe-tracker", NULL);
   g_object_set_data (G_OBJECT (self->swipeable), "captured-event-handler", NULL);
 
   g_clear_object (&self->touch_gesture);
@@ -601,7 +671,7 @@ hdy_swipe_tracker_get_property (GObject    *object,
 
   switch (prop_id) {
   case PROP_SWIPEABLE:
-    g_value_set_object (value, self->swipeable);
+    g_value_set_object (value, hdy_swipe_tracker_get_swipeable (self));
     break;
 
   case PROP_ENABLED:
@@ -676,17 +746,17 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
   object_class->set_property = hdy_swipe_tracker_set_property;
 
   /**
-   * HdySwipeTracker:widget:
+   * HdySwipeTracker:swipeable:
    *
    * The widget the swipe tracker is attached to. Must not be %NULL.
    *
-   * Since: 0.0.11
+   * Since: 1.0
    */
   props[PROP_SWIPEABLE] =
     g_param_spec_object ("swipeable",
                          _("Swipeable"),
                          _("The swipeable the swipe tracker is attached to"),
-                         GTK_TYPE_WIDGET,
+                         HDY_TYPE_SWIPEABLE,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
   /**
@@ -695,7 +765,7 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
    * Whether the swipe tracker is enabled. When it's not enabled, no events
    * will be processed. Usually widgets will want to expose this via a property.
    *
-   * Since: 0.0.11
+   * Since: 1.0
    */
   props[PROP_ENABLED] =
     g_param_spec_boolean ("enabled",
@@ -710,7 +780,7 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
    * Whether to reverse the swipe direction. If the swipe tracker is horizontal,
    * it can be used for supporting RTL text direction.
    *
-   * Since: 0.0.11
+   * Since: 1.0
    */
   props[PROP_REVERSED] =
     g_param_spec_boolean ("reversed",
@@ -725,7 +795,7 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
    * Whether to allow dragging with mouse pointer. This should usually be
    * %FALSE.
    *
-   * Since: 0.0.11
+   * Since: 1.0
    */
   props[PROP_ALLOW_MOUSE_DRAG] =
     g_param_spec_boolean ("allow-mouse-drag",
@@ -739,6 +809,69 @@ hdy_swipe_tracker_class_init (HdySwipeTrackerClass *klass)
                                     "orientation");
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
+
+  /**
+   * HdySwipeTracker::begin-swipe:
+   * @self: The #HdySwipeTracker instance
+   * @direction: The direction of the swipe
+   * @direct: %TRUE if the swipe is directly triggered by a gesture,
+   *   %FALSE if it's triggered via a #HdySwipeGroup
+   *
+   * This signal is emitted when a possible swipe is detected.
+   *
+   * The @direction value can be used to restrict the swipe to a certain
+   * direction.
+   *
+   * Since: 1.0
+   */
+  signals[SIGNAL_BEGIN_SWIPE] =
+    g_signal_new ("begin-swipe",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE,
+                  2,
+                  HDY_TYPE_NAVIGATION_DIRECTION, G_TYPE_BOOLEAN);
+
+  /**
+   * HdySwipeTracker::update-swipe:
+   * @self: The #HdySwipeTracker instance
+   * @progress: The current animation progress value
+   *
+   * This signal is emitted every time the progress value changes.
+   *
+   * Since: 1.0
+   */
+  signals[SIGNAL_UPDATE_SWIPE] =
+    g_signal_new ("update-swipe",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE,
+                  1,
+                  G_TYPE_DOUBLE);
+
+  /**
+   * HdySwipeTracker::end-swipe:
+   * @self: The #HdySwipeTracker instance
+   * @duration: Snap-back animation duration in milliseconds
+   * @to: The progress value to animate to
+   *
+   * This signal is emitted as soon as the gesture has stopped.
+   *
+   * Since: 1.0
+   */
+  signals[SIGNAL_END_SWIPE] =
+    g_signal_new ("end-swipe",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE,
+                  2,
+                  G_TYPE_INT64, G_TYPE_DOUBLE);
 }
 
 static void
@@ -751,30 +884,42 @@ hdy_swipe_tracker_init (HdySwipeTracker *self)
 
 /**
  * hdy_swipe_tracker_new:
- * @widget: a #GtkWidget to add the tracker on
+ * @swipeable: a #GtkWidget to add the tracker on
  *
  * Create a new #HdySwipeTracker object on @widget.
  *
  * Returns: the newly created #HdySwipeTracker object
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 HdySwipeTracker *
 hdy_swipe_tracker_new (HdySwipeable *swipeable)
 {
-  gpointer swipe_tracker;
-
-  g_return_val_if_fail (swipeable != NULL, NULL);
-
-  swipe_tracker = g_object_get_data (G_OBJECT (swipeable), "swipe-tracker");
-
-  g_return_val_if_fail (swipe_tracker == NULL, NULL);
+  g_return_val_if_fail (HDY_IS_SWIPEABLE (swipeable), NULL);
 
   return g_object_new (HDY_TYPE_SWIPE_TRACKER,
                        "swipeable", swipeable,
                        NULL);
 }
 
+/**
+ * hdy_swipe_tracker_get_swipeable:
+ * @self: a #HdySwipeTracker
+ *
+ * Get @self's swipeable widget.
+ *
+ * Returns: (transfer none): the swipeable widget
+ *
+ * Since: 1.0
+ */
+HdySwipeable *
+hdy_swipe_tracker_get_swipeable (HdySwipeTracker *self)
+{
+  g_return_val_if_fail (HDY_IS_SWIPE_TRACKER (self), NULL);
+
+  return self->swipeable;
+}
+
 /**
  * hdy_swipe_tracker_get_enabled:
  * @self: a #HdySwipeTracker
@@ -784,7 +929,7 @@ hdy_swipe_tracker_new (HdySwipeable *swipeable)
  *
  * Returns: %TRUE if @self is enabled
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 gboolean
 hdy_swipe_tracker_get_enabled (HdySwipeTracker *self)
@@ -802,7 +947,7 @@ hdy_swipe_tracker_get_enabled (HdySwipeTracker *self)
  * Set whether @self is enabled. When it's not enabled, no events will be
  * processed. Usually widgets will want to expose this via a property.
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 void
 hdy_swipe_tracker_set_enabled (HdySwipeTracker *self,
@@ -831,7 +976,7 @@ hdy_swipe_tracker_set_enabled (HdySwipeTracker *self,
  *
  * Returns: %TRUE is the direction is reversed
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 gboolean
 hdy_swipe_tracker_get_reversed (HdySwipeTracker *self)
@@ -849,7 +994,7 @@ hdy_swipe_tracker_get_reversed (HdySwipeTracker *self)
  * Set whether to reverse the swipe direction. If @self is horizontal,
  * can be used for supporting RTL text direction.
  *
- * Since: 0.0.11
+ * Since: 1.0
  */
 void
 hdy_swipe_tracker_set_reversed (HdySwipeTracker *self,
@@ -874,7 +1019,7 @@ hdy_swipe_tracker_set_reversed (HdySwipeTracker *self,
  *
  * Returns: %TRUE is mouse dragging is allowed
  *
- * Since: 0.0.12
+ * Since: 1.0
  */
 gboolean
 hdy_swipe_tracker_get_allow_mouse_drag (HdySwipeTracker *self)
@@ -892,7 +1037,7 @@ hdy_swipe_tracker_get_allow_mouse_drag (HdySwipeTracker *self)
  * Set whether @self can be dragged with mouse pointer. This should usually be
  * %FALSE.
  *
- * Since: 0.0.12
+ * Since: 1.0
  */
 void
 hdy_swipe_tracker_set_allow_mouse_drag (HdySwipeTracker *self,
@@ -913,6 +1058,16 @@ hdy_swipe_tracker_set_allow_mouse_drag (HdySwipeTracker *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ALLOW_MOUSE_DRAG]);
 }
 
+/**
+ * hdy_swipe_tracker_shift_position:
+ * @self: a #HdySwipeTracker
+ * @delta: the position delta
+ *
+ * Move the current progress value by @delta. This can be used to adjust the
+ * current position if snap points move during the gesture.
+ *
+ * Since: 1.0
+ */
 void
 hdy_swipe_tracker_shift_position (HdySwipeTracker *self,
                                   gdouble          delta)
@@ -926,3 +1081,32 @@ hdy_swipe_tracker_shift_position (HdySwipeTracker *self,
   self->progress += delta;
   self->initial_progress += delta;
 }
+
+void
+hdy_swipe_tracker_emit_begin_swipe (HdySwipeTracker        *self,
+                                    HdyNavigationDirection  direction,
+                                    gboolean                direct)
+{
+  g_return_if_fail (HDY_IS_SWIPE_TRACKER (self));
+
+  g_signal_emit (self, signals[SIGNAL_BEGIN_SWIPE], 0, direction, direct);
+}
+
+void
+hdy_swipe_tracker_emit_update_swipe (HdySwipeTracker *self,
+                                     gdouble          progress)
+{
+  g_return_if_fail (HDY_IS_SWIPE_TRACKER (self));
+
+  g_signal_emit (self, signals[SIGNAL_UPDATE_SWIPE], 0, progress);
+}
+
+void
+hdy_swipe_tracker_emit_end_swipe (HdySwipeTracker *self,
+                                  gint64           duration,
+                                  gdouble          to)
+{
+  g_return_if_fail (HDY_IS_SWIPE_TRACKER (self));
+
+  g_signal_emit (self, signals[SIGNAL_END_SWIPE], 0, duration, to);
+}
