@@ -16,12 +16,7 @@ G_BEGIN_DECLS
 
 #define HDY_TYPE_WINDOW_HANDLE (hdy_window_handle_get_type())
 
-G_DECLARE_DERIVABLE_TYPE (HdyWindowHandle, hdy_window_handle, HDY, WINDOW_HANDLE, GtkEventBox)
-
-struct _HdyWindowHandleClass
-{
-  GtkEventBoxClass parent_class;
-};
+G_DECLARE_FINAL_TYPE (HdyWindowHandle, hdy_window_handle, HDY, WINDOW_HANDLE, GtkEventBox)
 
 GtkWidget *hdy_window_handle_new (void);
 
